/* 
 * botbinds.c -- part of botnetop.mod
 *
 * $Id: botbinds.c,v 1.6 2001/02/13 18:23:24 sup Exp $
 */
/*
 * Copyright (C) 2000, 2001  Teemu Hjelt <temex@iki.fi>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static int bnop_linkop(char *bot, char *via)
{
  struct chanset_t *chan = NULL;

  if ((bop_linkop) && ((!egg_strcasecmp(bot, botnetnick)) || (!egg_strcasecmp(via, botnetnick)))) {
    for (chan = chanset; chan; chan = chan->next) {
      if ((ismember(chan, botname)) && (!isop(botname, chan)))
        bnop_reqop(chan->dname);
    }
  }
  return 0;
}

static int bnop_doiwantops(char *bot, char *code, char *par)
{
  int i;
  char *chname, *fromnick, data[256];
  struct chanset_t *chan = NULL;
  struct delay_t *d = NULL;
  struct userrec *u;
  memberlist *m;

  chname = newsplit(&par);
  fromnick = newsplit(&par);
  if ((!chname[0]) || (!fromnick[0]))
    return 0;
  if ((chan = findchan_by_dname(chname)) && (ismember(chan, botname)) && (!isop(botname, chan))) {
    u = get_user_by_handle(userlist, bot);
    if ((m = ismember(chan, fromnick)) && (!chan_issplit(m)) && (isop(fromnick, chan)) && (matchattr(u, "o|o", chan->dname))) {
      i = nextbot(bot);
      if (i >= 0) {
        if (bop_maxreq) {
          if (!(d = find_delay(chname)))
            d = add_delay(chname, bot);
          d->reqs++; 
          if (d->reqs >= bop_maxreq + 1) {
            if (d->reqs == bop_maxreq + 1)
              d->resptime = now + 30;
            return 0;
          }
          d->resptime = now + 30;
        }
        egg_snprintf(data, sizeof data, "yesiwantops %s %s %s", chan->dname, botname, strchr("~+-^=", botuserhost[0]) ? botuserhost + 1 : botuserhost);
        botnet_send_zapf(i, botnetnick, bot, data);
        if (bop_log >= 2)
          putlog(LOG_MISC, "*", "botnetop.mod: requested ops from %s on %s", bot, chan->dname);
      }
    }
  }
  return 0;
}

static int bnop_botwantsops(char *bot, char *code, char *par)
{
  char *chname, *fromnick, *fromhost, s[UHOSTLEN];
  struct chanset_t *chan = NULL;
  struct userrec *u;
  memberlist *m;

  chname = newsplit(&par);
  fromnick = newsplit(&par);
  fromhost = newsplit(&par);
  if ((!chname[0]) || (!fromnick[0]))
    return 0;
  if ((!(chan = findchan_by_dname(chname))) || (!ismember(chan, botname)) || (!isop(botname, chan)))
    return 0;
  u = get_user_by_handle(userlist, bot);
  if ((m = ismember(chan, fromnick)) && (!chan_issplit(m)) && (matchattr(u, "b|-", chan->dname)) &&
      (matchattr(u, "o|o", chan->dname)) && (!matchattr(u, "d|d", chan->dname))) {
    egg_snprintf(s, sizeof s, "%s!%s", m->nick, m->userhost);
    u = get_user_by_host(s);
    if (!matchattr(u, "o|o", chan->dname)) {
      if ((!fromhost[0]) || (!bop_addhost))
        return 0;
      if (strchr("~^+=-", m->userhost[0]))
        egg_snprintf(s, sizeof s, "*!%s%s", strict_host ? "?" : "", m->userhost + 1);
      else
        egg_snprintf(s, sizeof s, "*!%s", m->userhost);
      addhost_by_handle(bot, s);
      putlog(LOG_MISC, "*", "botnetop.mod: added host %s to %s", s, bot);
    }
    if (!isop(fromnick, chan))
      add_mode(chan, '+', 'o', fromnick);
    else if ((bop_osync) && (isop(fromnick, chan)))
      dprintf(DP_MODE, "MODE %s +o %s\n", chan->dname, fromnick);
    if (bop_log >= 2) {
      if (egg_strcasecmp(fromnick, bot))
        putlog(LOG_MISC, "*", "botnetop.mod: gave ops to %s (using nick %s) on %s", bot, fromnick, chan->dname);
      else
        putlog(LOG_MISC, "*", "botnetop.mod: gave ops to %s on %s", bot, chan->dname);
    }
  }
  return 0;
}

static int bnop_botwantsin(char *bot, char *code, char *par)
{
  int i, bans = 0;
  char *chname, *fromnick, *fromhost, data[256];
  struct chanset_t *chan = NULL;
  struct who_t *w = NULL;
  struct userrec *u;
  masklist *b;

  chname = newsplit(&par);
  fromnick = newsplit(&par);
  fromhost = newsplit(&par);
  if ((!chname[0]) || (!(chan = findchan_by_dname(chname))) || (!isop(botname, chan)))
    return 0;
  u = get_user_by_handle(userlist, bot);
  if ((matchattr(u, "b|-", chan->dname)) && (matchattr(u, "fo|fo", chan->dname))) {
    i = nextbot(bot);
    if (i < 0)
      return 0;
    if (!egg_strcasecmp(code, "wantkey")) {
      if (chan->channel.mode & CHANKEY) {
        egg_snprintf(data, sizeof data, "thekey %s %s", chan->dname, chan->channel.key);
        botnet_send_zapf(i, botnetnick, bot, data);
        if (bop_log >= 1)
          putlog(LOG_MISC, "*", "botnetop.mod: gave key for %s to %s", chan->dname, bot);
      }
    } else if (!egg_strcasecmp(code, "wantinvite")) {
      if (chan->channel.mode & CHANINV) {
        if (!fromnick[0])
          return 0;
        if ((bop_icheck) && (fromhost[0])) {
          if (!(w = find_who(fromnick)))
            w = add_who(fromnick, chname, bot, fromhost);
          dprintf(DP_SERVER, "USERHOST %s\n", fromnick);
        } else {
          dprintf(DP_SERVER, "INVITE %s %s\n", fromnick, chan->dname);
          if (bop_log >= 1) {
            if (egg_strcasecmp(fromnick, bot))
              putlog(LOG_MISC, "*", "botnetop.mod: invited %s (using nick %s) to %s", bot, fromnick, chan->dname);
            else
              putlog(LOG_MISC, "*", "botnetop.mod: invited %s to %s", bot, chan->dname);
          }
        }
      }
    } else if (!egg_strcasecmp(code, "wantlimit")) {
      if (chan->channel.maxmembers > -1) {
        add_mode(chan, '+', 'l', int_to_base10(chan->channel.members + 1));
        if (bop_log >= 1)
          putlog(LOG_MISC, "*", "botnetop.mod: raised limit on %s as requested by %s", chan->dname, bot);
      }
    } else if (!egg_strcasecmp(code, "wantunban")) {
      if (!fromhost[0])
        return 0;
      for (b = chan->channel.ban; b->mask[0]; b = b->next) {
        if (wild_match(b->mask, fromhost)) {
          add_mode(chan, '-', 'b', b->mask);
          bans++;
        }
      }
      if ((bop_log >= 1) && (bans > 0))
        putlog(LOG_MISC, "*", "botnetop.mod: unbanned %s on %s", bot, chan->dname);
    }
  }
  return 0;
}

static int bnop_reqtmr(char *bot, char *code, char *par)
{
  char *chname;
  struct chanset_t *chan = NULL;
  struct delay_t *d = NULL;
  struct userrec *u;

  chname = newsplit(&par);
  if ((chname[0]) && (chan = findchan_by_dname(chname))) {
    u = get_user_by_handle(userlist, bot);
    if ((matchattr(u, "b|-", chan->dname)) && (matchattr(u, "o|o", chan->dname)) && (!matchattr(u, "d|d", chan->dname))) {
      if ((bop_delay) && (!lowbots(chan))) {
        if (!(d = find_delay(chname)))
          d = add_delay(chname, bot);
        d->reqtime = now + (random() % bop_delay) + 2;
        strncpyz(d->bot, bot, sizeof d->bot);
      } else
        bnop_askbot(bot, chan->dname);
    }
  }
  return 0;
}

static int bnop_joinkey(char *bot, char *code, char *par)
{
  char *chname, *key;
  struct chanset_t *chan = NULL;
  struct needs_t *n = NULL;

  chname = newsplit(&par);
  key = newsplit(&par);
  if ((chname[0]) && (key[0]) && (chan = findchan_by_dname(chname)) && (!ismember(chan, botname))) {
    if (!(n = find_needs(chname)))
      n = add_needs(chname);
    if (n->kjoin)
      return 0;
    dprintf(DP_SERVER, "JOIN %s %s\n", chan->dname, key);
    n->kjoin = 1;
    n->kjoin_time = now;
  }
  return 0;
}

static cmd_t botnetop_bot[] =
{
  {"doyawantops", "", (Function) bnop_doiwantops, "bop_doiwantops"},
  {"yesiwantops", "", (Function) bnop_botwantsops, "bop_botwantsops"},
  {"reqops", "", (Function) bnop_reqtmr, "bop_reqtmr"},
  {"wantkey", "", (Function) bnop_botwantsin, "bop_botwantsin"},
  {"wantinvite", "", (Function) bnop_botwantsin, "bop_botwantsin"},
  {"wantlimit", "", (Function) bnop_botwantsin, "bop_botwantsin"},
  {"wantunban", "", (Function) bnop_botwantsin, "bop_botwantsin"},
  {"thekey", "", (Function) bnop_joinkey, "bop_joinkey"},
  {0, 0, 0, 0}
};

static cmd_t botnetop_link[] =
{
  {"*", "", (Function) bnop_linkop, "bop_linkop"},
  {0, 0, 0, 0}
};
