/*
 * botnetop.c -- part of botnetop.mod
 *  
 * $Id: botnetop.c,v 1.8 2001/02/13 18:23:24 sup Exp $
 */
/*
 * Copyright (C) 2000, 2001  Teemu Hjelt <temex@iki.fi>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#define MODULE_NAME "botnetop"
#define MAKING_BOTNETOP
#define MODULE_VERSION "1.0.1"
#include "../module.h"
#include "../irc.mod/irc.h"
#include "../server.mod/server.h"
#include "../channels.mod/channels.h"

#undef global
static Function *global = NULL, *irc_funcs = NULL, *server_funcs = NULL, *channels_funcs = NULL;

static int strict_host = 0;
static int bop_delay = 20;
static int bop_maxreq = 3;
static int bop_modeop = 0;
static int bop_linkop = 1;
static int bop_icheck = 0;
static int bop_osync = 0;
static int bop_addhost = 0;
static int bop_log = 2;
static struct delay_t *start_delay = NULL;
static struct who_t *start_who = NULL;
static struct needs_t *start_needs = NULL;

#include "botnetop.h"
#include "misc.c"
#include "botcmds.c"
#include "ircbinds.c"
#include "botbinds.c"
#include "needbinds.c"
#include "tclcmds.c"
#include "delay.c"
#include "who.c"
#include "needs.c"

static void botnetop_hook_secondly()
{
  check_delay();
  check_who();
  check_needs();
}

static void botnetop_hook_hourly()
{
  struct chanset_t *chan = NULL;
  struct delay_t *d = NULL, *dnext = NULL;
  struct needs_t *n = NULL, *nnext = NULL;

  for (d = start_delay; d; d = dnext) {
    dnext = d->next;
    if (!(chan = findchan_by_dname(d->chan)))
      del_delay(d);
  }
  for (n = start_needs; n; n = nnext) {
    nnext = n->next;
    if (!(chan = findchan_by_dname(n->chan)))
      del_needs(n);
  }
}

static int botnetop_expmem()
{
  return delay_expmem() + who_expmem() + needs_expmem();
}

static void botnetop_report(int idx, int details)
{
  int dtot = 0;
  int wtot = 0;
  int ntot = 0;
  struct delay_t *d = NULL;
  struct who_t *w = NULL;
  struct needs_t *n = NULL;

  for (d = start_delay; d; d = d->next)
    dtot++;
  for (w = start_who; w; w = w->next)
    wtot++;
  for (n = start_needs; n; n = n->next)
    ntot++;
  if (details) {
    dprintf(idx, "    Version: %s\n", MODULE_VERSION);
    dprintf(idx, "    Tcl commands: bop_reqop, bop_letmein\n");
    dprintf(idx, "    %d delay record%s using %d bytes\n", dtot, dtot == 1 ? "" : "s", delay_expmem());
    dprintf(idx, "    %d who record%s using %d bytes\n", wtot, wtot == 1 ? "" : "s", who_expmem());
    dprintf(idx, "    %d needs record%s using %d bytes\n", ntot, ntot == 1 ? "" : "s", needs_expmem());
    dprintf(idx, "    botnetop.mod is using %d bytes\n", botnetop_expmem());
  }
}

static tcl_ints botnetop_tcl_ints[] =
{
  {"strict-host", &strict_host, 0},
  {"bop_delay", &bop_delay, 0},
  {"bop_maxreq", &bop_maxreq, 0},
  {"bop_modeop", &bop_modeop, 0},
  {"bop_linkop", &bop_linkop, 0},
  {"bop_icheck", &bop_icheck, 0},
  {"bop_osync", &bop_osync, 0},
  {"bop_addhost", &bop_addhost, 0},
  {"bop_log", &bop_log, 0},
  {0, 0, 0}
};

static char *botnetop_close()
{
  del_hook(HOOK_SECONDLY, (Function) botnetop_hook_secondly);
  del_hook(HOOK_HOURLY, (Function) botnetop_hook_hourly);
  rem_builtins(H_join, botnetop_join);
  rem_builtins(H_mode, botnetop_mode);
  rem_builtins(H_raw, botnetop_raw);
  rem_builtins(H_link, botnetop_link);
  rem_builtins(H_bot, botnetop_bot);
  rem_builtins(H_need, botnetop_need);
  rem_tcl_ints(botnetop_tcl_ints);
  rem_tcl_commands(botnetop_tcl_cmds);
  delay_free_mem();
  who_free_mem();
  needs_free_mem();  
  module_undepend(MODULE_NAME);
  return NULL;
}

char *botnetop_start();

static Function botnetop_table[] =
{
  (Function) botnetop_start,
  (Function) botnetop_close,
  (Function) botnetop_expmem,
  (Function) botnetop_report,
};

char *botnetop_start(Function * global_funcs)
{
  global = global_funcs;
  module_register(MODULE_NAME, botnetop_table, 1, 0);
  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
    if (!module_depend(MODULE_NAME, "eggdrop", 105, 4)) {
      module_undepend(MODULE_NAME);
      return "This module requires eggdrop1.5.4 or later";
    }
  }
  if (!(irc_funcs = module_depend(MODULE_NAME, "irc", 1, 0))) {
    module_undepend(MODULE_NAME);
    return "You need the irc module to use the botnetop module.";
  }
  if (!(server_funcs = module_depend(MODULE_NAME, "server", 1, 0))) {
    module_undepend(MODULE_NAME);
    return "You need the server module to use the botnetop module.";
  }
  if (!(channels_funcs = module_depend(MODULE_NAME, "channels", 1, 0))) {
    module_undepend(MODULE_NAME);
    return "You need the channels module to use the botnetop module.";
  }
  add_hook(HOOK_SECONDLY, (Function) botnetop_hook_secondly);
  add_hook(HOOK_HOURLY, (Function) botnetop_hook_hourly);
  add_builtins(H_join, botnetop_join);
  add_builtins(H_mode, botnetop_mode);
  add_builtins(H_raw, botnetop_raw);
  add_builtins(H_link, botnetop_link);
  add_builtins(H_bot, botnetop_bot);
  add_builtins(H_need, botnetop_need);
  add_tcl_ints(botnetop_tcl_ints);
  add_tcl_commands(botnetop_tcl_cmds);
  return NULL;
}
