/*
 * delay.c -- part of botnetop.mod
 *
 * $Id: delay.c,v 1.4 2001/02/13 18:23:24 sup Exp $
 */
/* 
 * Copyright (C) 2000, 2001  Teemu Hjelt <temex@iki.fi>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static struct delay_t *find_delay(char *chan)
{
  struct delay_t *d = NULL;

  for (d = start_delay; d; d = d->next)
    if (!rfc_casecmp(d->chan, chan))
      return d;
  return NULL;
}

static struct delay_t *add_delay(char *chan, char *bot)
{
  struct delay_t *d = NULL;

  d = (struct delay_t *) nmalloc(sizeof(struct delay_t));
  if (!d)
    return NULL;
  d->next = start_delay;
  start_delay = d;
  strncpyz(d->chan, chan, sizeof d->chan);
  strncpyz(d->bot, bot, sizeof d->bot);
  d->reqtime = 0;
  d->resptime = 0;
  d->reqs = 0;
  putlog(LOG_DEBUG, "*", "botnetop.mod: new delay record created for %s (address: %u)", chan, d);
  return d;
}

static void del_delay(struct delay_t *delay)
{
  struct delay_t *d = NULL, *old = NULL;

  for (d = start_delay; d; old = d, d = d->next) {
    if (d == delay) {
      if (old)
        old->next = d->next;
      else
        start_delay = d->next;
      putlog(LOG_DEBUG, "*", "botnetop.mod: delay record removed from %s (address: %u)", d->chan, d);
      nfree(d);
      break;
    }
  }
}

static void check_delay()
{
  struct delay_t *d = NULL;

  for (d = start_delay; d; d = d->next) {
    if ((d->reqtime) && (d->reqtime <= now)) {
      d->reqtime = 0;
      bnop_askbot(d->bot, d->chan);
    }
    if ((d->resptime) && (d->resptime <= now)) {
      d->resptime = 0;
      d->reqs = 0;
    }
  }
}

static void delay_free_mem()
{
  struct delay_t *d = NULL, *dnext = NULL;

  for (d = start_delay; d; d = dnext) {
    dnext = d->next;
    nfree(d);
  }
  start_delay = NULL;
}

static int delay_expmem()
{
  int size = 0;
  struct delay_t *d = NULL;

  for (d = start_delay; d; d = d->next)
    size += sizeof(struct delay_t);
  return size;
}
